'use strict';

var assert = require('assert');
var _ = require('lodash');
var vk = require('../index');
var search = vk.search;

describe('tweenSearch', function () {

  describe('deterministic', function () {

    it('returns null for an empty list', function () {
      var result = search.tween([], 0);
      assert.equal(result, null);
    });

    it('returns the only element for a single-element list', function () {
      var keyframe = { stop: 0.25 };

      var result = search.tween([keyframe], 0);
      assert.equal(result.length, 1);
      assert.equal(result[0], keyframe);

      result = search.tween([keyframe], 0.5);
      assert.equal(result.length, 1);
      assert.equal(result[0], keyframe);

      result = search.tween([keyframe], 1);
      assert.equal(result.length, 1);
      assert.equal(result[0], keyframe);
    });

    it('returns the low element when time < low.stop', function () {
      var keyframes = [{ stop: 0.1 }, { stop: 0.5 }, { stop: 0.9 }];

      var result = search.tween(keyframes, 0.05);
      assert.equal(result.length, 1);
      assert.equal(result[0], keyframes[0]);

      result = search.tween(keyframes, 0.1);
      assert.equal(result.length, 2);
      assert.equal(result[0], keyframes[0]);
    });

    it('returns the high element when time >= high.stop', function () {
      var keyframes = [{ stop: 0.1 }, { stop: 0.5 }, { stop: 0.9 }];

      var result = search.tween(keyframes, 0.9);
      assert.equal(result.length, 1);
      assert.equal(result[0], keyframes[2]);

      result = search.tween(keyframes, 0.95);
      assert.equal(result.length, 1);
      assert.equal(result[0], keyframes[2]);
    });

    it('returns the correct range in a long list of keyframes', function () {
      var keyframes = [];
      var size = 1000;
      for (var i = 1; i < size; i++) {
        keyframes.push({ stop: i / size });
      }

      for (var _i = 0; _i <= 2 * size; _i++) {
        var time = _i / (2 * size);
        var result = search.tween(keyframes, time);

        if (result.length == 1) {
          if (time < 0.5) {
            assert(time <= result[0].stop, 'time is le low end');
          } else {
            assert(time >= result[0].stop, 'time is ge high end');
          }
        } else if (result.length == 2) {
          assert(result[0].stop <= time && time < result[1].stop, 'time is between keyframe pair');
        } else {
          assert(false, 'list lengths should be 1 or 2');
        }
      }
    });
  });

  it('finds keyframes in O(lg(n)) time', function () {
    var maxSize = 1e3;
    var skip = 100;

    // Generate lists of size [skip, maxSize]

    var _loop = function _loop(size) {
      var maxCompares = 0;
      var keyframes = [];

      for (var i = 0; i < size; i++) {
        keyframes.push({ stop: Math.random() });
      }

      var _loop2 = function _loop2(_i2) {
        var compares = 0;
        var time = _i2 / size;

        search.tween(keyframes, time, function (a, b, time) {
          maxCompares = Math.max(maxCompares, ++compares);

          if (a.stop <= time && time < b.stop) {
            return 0;
          } else {
            if (time < a.stop) {
              return -1;
            } else {
              return 1;
            }
          }
        });

        assert(maxCompares <= Math.floor(Math.log2(size)) + 1, 'maxCompares <= log2(list_size) + 1');
      };

      for (var _i2 = 0; _i2 < size; ++_i2) {
        _loop2(_i2);
      }
    };

    for (var size = skip; size <= maxSize; size += skip) {
      _loop(size);
    }
  });
});

function timeIt(n, callback) {
  var start = process.hrtime();

  for (var i = 0; i < n; ++i) {
    callback();
  }
  var timeTuple = process.hrtime(start);

  return timeTuple[0] * 1e9 + timeTuple[1];
}
//# sourceMappingURL=search_spec.js.map